#######################################################
# Tool-functions for 'Test4IWS.R' 
# Encoded by B. Gauthier. 
# Work supported by the project ANR-2011-IS01-001-01 DESIRE (DESIgns for spatial Random fiElds).
########################################################

########################################################
# Regular middle point quadrature on a segment [a,b].
# Name: "DisInterEquiMid";
# Input : 
# a and b .... for the segment [a,b].
# ndys ....... number of point for the discretisation.
# Output : a vector containing the coordinates of the discretizations points.
##########################################################
DisInterEquiMid<-function(a,b,ndys){		
ldys<-(b-a)/ndys
xdysC<-seq(a,b,,ndys+1)
xdysC[-(ndys+1)]+ldys/2  
}



#####################################################################
# Construction of the Grid covariance matrix (column by column, save time for big matrix).
# Name: "GridMatCov".
# Input:
# GridP       ...    A matrix of grid point (rows are the points coordinates).
# KKernel     ...    The covariance kernel to be used, (a R function with 2 entries).  
# Output: The covariance matrix of the grid points.
######################################################################
GridMatCov<-function(GridP,KKernel){
if (nrow(GridP)>1){	
KKKernel<-function(x,y) KKernel(x,y)	
  ntGr<-nrow(GridP)
  KKGr<-matrix(0,ntGr,ntGr)
  for (i in 1:(ntGr-1)){
 	KKGr[i,i:ntGr]<-apply(as.matrix(GridP[i:ntGr,]),1,KKKernel,y=GridP[i,]) # as.matrix is for the case where GridP[k,] is a vector
	KKGr[(i+1):ntGr,i]<-KKGr[i,(i+1):ntGr]  }
  KKGr[ntGr,ntGr]<-KKernel(GridP[ntGr,],GridP[ntGr,]) }
else { KKGr<-KKernel(GridP,GridP) }  
KKGr } # end GridMatCov

#####################################################################
# Rectangular grid construction from the list of axes discretization. 
# Name: "GridRectCons".
# Input:
# GridRectL    ...   A list whose components are the vectors of the points coordinates for each axis.
# Output: A list with 
# .$Grid       ...   A matrix whose rows are the grid points coordinates.
# .$Dim        ...   The dimension of where the points of the grid lie (i.e., the number of column in xxx$Grid). 
# .$TotalPts   ...   The number of points in the grid (i.e., the number of row in xxx$Grid). 
# .$Specif     ...   The number of points on each axis.
######################################################################

GridRectCons<-function(GridRectL){
GrS<-summary(GridRectL)
GrSM<-as.matrix(GrS)

Vndys<-as.integer(GrSM[,1]) #Vector of the number of points in each dimension
 
ntGr<-prod(Vndys)  # Grid size (total number of point)
DimGr<-length(Vndys)  # Number of dimension

GridConsRec<-function(GridRectL,DimGr,Vndys){
	if (DimGr==1){ GridQ<-matrix(GridRectL[[1]],Vndys[1]) } else{
		Rz<-GridConsRec(GridRectL,DimGr-1,Vndys)
		P<-Rz[[1]]
		noob<-Rz[[2]]+1
		cndys<-Vndys[noob]
		cdys<-GridRectL[[noob]]
		cNr<-nrow(P)
		MP<-P
		ww<-rep(cdys[1],cNr)
		if (cndys>1) {
			for (k in 2:cndys) {
				MP<-rbind(MP,P)
				ww<-c(ww,rep(cdys[k],cNr)) }
	}
	GridQ<-cbind(MP,ww)
	}
list(GridQ,DimGr)
}# end GridConsRec
GridP<-GridConsRec(GridRectL,DimGr,Vndys)[[1]]
list(Grid=GridP,Dim=DimGr,TotalPts=ntGr,Specif=Vndys)
} # end GridRectCons



###################################################
# Multiplication by a diagonal matrix (diagonal matrix on the left hand).
# Name: "L.Diago.Mat.Mult".
# Input:
# diagElm ...... the vector of the diagonal elements.
# Mat .......... a matrix.
# Output: the matrix diag(diagElm)%*%Mat.
####################################################
   
L.Diago.Mat.Mult<-function(diagElm,Mat){
diagElm*Mat
}



###################################################
# Direct computation of the IMSE (quadrature approximation).
# Name: "Direct.IMSE". 
# Input:
# Plan .......... The design of experiments, a matrix whose rows are the points coordinates.
# Q.Gr .......... Matrix of the quadrature points (rows are the points coordinates).
# Q.Wg .......... Vector of the quadrature weights.
# KKernel ....... The kernel (a bivariate function). 
# With.trend .... TRUE for models with trend and FALSE for model without trend.     
# Trend ......... Optional, only for model with (linear) trend. 
#                 The (vector valued) function of the trend functions, i.e. a function of the type 
#                 Trend<-function(x) c(1,x[1],x[1]^2).      
# Output: the value of the IMSE.
####################################################
Direct.IMSE<-function(Plan,Q.Gr,Q.Wg,KKernel,With.trend,Trend){
 
MatKm1<-solve(GridMatCov(Plan,KKernel))

if (With.trend==FALSE){ # IMSE without trend
funkV<-function(x){
	kplX<-apply(as.matrix(Plan),1,KKernel,y=x)
	KKernel(x,x)-t(kplX)%*%MatKm1%*%kplX }
sum(apply(Q.Gr,1,funkV)*Q.Wg)
}
else { # IMSE with trend

MatReg<-t(apply(Plan,1,Trend))
Wym1<-solve(t(MatReg)%*%MatKm1%*%MatReg)

funkV<-function(x){
kplX<-apply(Plan,1,KKernel,y=x)
GKtlX<-Trend(x)-t(MatReg)%*%MatKm1%*%kplX
KKernel(x,x)-t(kplX)%*%MatKm1%*%kplX+t(GKtlX)%*%Wym1%*%GKtlX }
sum(apply(Q.Gr,1,funkV)*Q.Wg) }
} # end Direct.IMSE

#################################################################
#################################################################
# Design perturbation
####################
# Proximity, one-point at a time, perturbation
# Name: "GrPtProxPerturb". 
# Input : 
# Pt2Pert ......... index of the design point to perturbe (1st, 2nd,...),
# CurDesI ......... indices of the quadrature-design to perturbe
# DesTai .......... design size
# Nb4Prox ......... number of perturbed points
# ProxyMatGr ...... proximity matrix
# Output: a matrix which row are the new designs.  
###################################################################

GrPtProxPerturb<-function(Pt2Pert,CurDesI,DesTai,Nb4Prox,ProxyMatGr){	 
# Point perturbations from Proxy Matrix   
	CurProxI<-ProxyMatGr[1:(Nb4Prox+DesTai-1),CurDesI[Pt2Pert]]
	CurProxI<-setdiff(CurProxI,CurDesI)[1:Nb4Prox] # leave out the new points which are already in the design 
# Designs with new points 
Nniou<-length(CurProxI)
MpDes<-matrix(rep(CurDesI,Nniou),DesTai,Nniou)
MpDes[Pt2Pert,]<-CurProxI
t(MpDes)  # lines of the returned matrix are the new designs
} # end GrPtProxPerturb


###################################################################
# Proximity + full random perturbation. 
# Name: "ZoupPerturb".
# Input:
# ... idem 'GrPtProxPerturb' + 
# Nrand ....... number of random points.
# Nq .......... number of quadrature points.
# Output: a matrix which row are the new designs.
####################################################################
ZoupPerturb<-function(Pt2Pert,CurDesI,DesTai,Nprox,Nrand,Nq,ProxyMatGr){	 
# Point perturbations from Proxy Matrix   
	CurProxI<-ProxyMatGr[1:(Nprox+DesTai-1),CurDesI[Pt2Pert]]
	CurProxI<-setdiff(CurProxI,CurDesI)[1:Nprox] # leave out the new points which are already in the design 
	OccuPtI<-c(CurDesI,CurProxI)
# Random perturbations from Cov Matrix and weights 
NiouRand<-sample((1:Nq)[-OccuPtI],Nrand,replace=FALSE)

NiouDesPtI<-c(CurProxI,NiouRand)

MpDes<-matrix(rep(CurDesI,Nprox+Nrand),DesTai,Nprox+Nrand)
MpDes[Pt2Pert,]<-NiouDesPtI

t(MpDes)  # lines of the returned matrix are the new designs
} # end ZoupPerturb

#################################################################
## OPTIMIZATION
#################################################################
# Local descent with one point at a time perturbation (random order). 
# Name: "BibiteDescent"
# Input: 
# IniId ........ initial design index set. 
# Krit ......... the criterion to optimize.
# Nprox ........ number of proximity perturbations.
# ProyMatGr .... proximity matrix.
# Output: a list with
# $Des ......... the obtained design.
# $Info ........ a vector containing the number of cycles and the number of improvements.
# $Val ......... Score for the best design. 
#################################################################
BibiteDescent<-function(IniId,Krit,Nprox,ProxyMatGr){
  CurDes<-IniId              # Curent design for IMSE   
  np<-length(IniId)          # Size of the design 
  CurBest<-Krit(CurDes)      # Score for the curent design 
  #ValRec<-CurBest
  NbCycle<-0                 # Number of optimization cycles         
  NbImprov<-0                # Number of improvements

  Improv<-TRUE  
     while (Improv){
     NbCycle<-NbCycle+1
     OrderMouv<-sample(1:np,np,replace=FALSE)
     Improv<-FALSE	
     for (k in 1:np){
     	NewDes<-GrPtProxPerturb(OrderMouv[k],CurDes,np,Nprox,ProxyMatGr)
     	NewVal<-apply(NewDes,1,Krit)
     	Chaleng<-which.min(NewVal)
     	  if( NewVal[Chaleng]<CurBest ){
     	         CurDes<-NewDes[Chaleng,]
     	         CurBest<-NewVal[Chaleng]
     	         #ValRec<-c(ValRec,CurBest)
     	         Improv<-TRUE
     	         NbImprov<-NbImprov+1  }    	
     }
     }       
list(Info=c(NbCycle,NbImprov),Des=CurDes,Val=CurBest)  
}



####################################################################
# Hybrid-ESE algorithm
# Name: "GridEseOptim".
# Input:
# IniId ......... index set of the initial design
# Krit .......... the criterion to optimize
# Nprox ......... number of proximity perturbations
# Nrand ......... number of random perturbation
# ProxyMatGr .... proximity matrix
# innerIt ....... consecutive cycles without temperature update
# it ............ number of temperature update
# Tol ........... tolerance for the acceptation of a temperature update (e.g., a small positive real, as 1e-8)
# Nq ............ number of quadrature points
# Output: a list with
# $Design ....... the obtained design
# $ESE.Design ... the design after ESE (without the final local descent)
# $Eval ......... number of criterion evaluation during ESE
# $ESE.When ..... when ESE has found its best design
# $Temp ......... temperature recording
# $LocD.Info .... information for the final local descent
#####################################################################
GridEseOptim<-function(IniId,Krit,Nprox,Nrand,ProxyMatGr,innerIt,it,Tol,Nq){  	 

  NbPert<-Nprox+Nrand
  CurDes<-IniId                      # Curent design
  np<-length(IniId)                  # Size of the design 
  bestDes<-CurDes                    # Best design (initialization)
  CurSqr<-Krit(CurDes)               # Score of the curent design 
  bestVal<-CurSqr                    # Best (i.e. minimun) IMSE
  Temperature<-0.005*CurSqr          # Initial Temperature
  NbEval<-1                          # Number of IMSE evaluation
  WhenBest<-NbEval                   # Indicate when the best value has been found
  TempRec<-Temperature               # Temperature recording
  
  for (q in 1:it){
     oldDes<-bestDes  ; oldVal<-bestVal                           
     ni<-0 ; na<-0 ; 			         # ni = number of improvements ; na = number of acceptances 
     for (count in 1:innerIt){           # begin the inner loop
     OrderMouv<-sample(1:np,np,replace=FALSE)
     for (k in 1:np){  	
     MpDes<-ZoupPerturb(OrderMouv[k],CurDes,np,Nprox,Nrand,Nq,ProxyMatGr)    # Design perturbations
     TryValS<-apply(MpDes,1,Krit)                                            # New designs IMSE
     NbEval<-NbEval+NbPert
     
     kb<-which.min(TryValS)
     kbDes<-MpDes[kb,]
     kbVal<-TryValS[kb]
     Delta<-kbVal-CurSqr
                                                
     if(Delta<=0){ # Always go if improvement
     	   CurDes<-kbDes
           CurSqr<-kbVal
           na<-na+1           
             if(CurSqr<=bestVal){  
                  bestDes<-CurDes            
                  bestVal<-CurSqr 
                  WhenBest<-NbEval 
                  ni<-ni+1 } }   # end (if Delta<=0)
     else{ # Stochastic acceptance rule applied only to the best random perturbations 
     kbRand<-which.min(TryValS[-(1:Nprox)]) # Best rand value
     kbRandDes<-MpDes[Nprox+kbRand,]
     kbRandVal<-TryValS[Nprox+kbRand]
     Delta4Rand<-kbRandVal-CurSqr
      
      if(Delta4Rand<=(Temperature*runif(1))){
     	   CurDes<-kbRandDes
           CurSqr<-kbRandVal
           na<-na+1 
           }   
                  } # end else               
  } # end for one OrderMouv
  } # end inner loop
  va<-na/(innerIt*np)    # va<-acceptance ratio
  vi<-ni/(innerIt*np)    # vi<-improvement ratio               
# Temperature update
  if (abs(bestVal-oldVal)<Tol){
              if(va>=0.1 & vi<=va)
                  {Temperature<-0.8*Temperature}
                       else {if (va>=0.1 & vi==va){  } 
                                      else {Temperature<-Temperature/0.8} }  }
      else { 
      if (va<=0.1){Temperature<-Temperature/0.7}
              else {Temperature<-Temperature*0.9} }
      TempRec<-c(TempRec,Temperature)                                    
                                               } # end outer loop
# Local descent (to finish)                                               
ESEraff<-BibiteDescent(bestDes,Krit,Nprox,ProxyMatGr)  
bestVal<-ESEraff$Val
bestDes<-ESEraff$Des                                            
EndLocDesc<-BibiteDescent(CurDes,Krit,Nprox,ProxyMatGr)              
if (EndLocDesc$Val<=bestVal){ FbestDes<-EndLocDesc$Des } else{ FbestDes<-bestDes }

                                               
list(Design=FbestDes,ESE.Design=bestDes,ESE.Eval=NbEval,ESE.When=WhenBest,Temp=TempRec,LocD.Info=EndLocDesc$Info)
} # end GridEseOptim


####################################################################################
####################################################################################
######################## BONUS 
####################################################################################

# With block inversion of the covariance matrix
# (use the fact that perturbed designs differ from the current design by only one point)
#################################### 
# Design perturbation (see ZoupPerturb)
ZoupPerturb4block<-function(Pt2Pert,CurDesI,DesTai,Nprox,Nrand,Nq,ProxyMatGr){	 
# Point perturbations from Proxy Matrix   
	CurProxI<-ProxyMatGr[1:(Nprox+DesTai-1),CurDesI[Pt2Pert]]
	CurProxI<-setdiff(CurProxI,CurDesI)[1:Nprox] # leave out the new points which are already in the design 
	OccuPtI<-c(CurDesI,CurProxI)
# Random perturbations from Cov Matrix and weights 
NiouRand<-sample((1:Nq)[-OccuPtI],Nrand,replace=FALSE)

NiouDesPtI<-c(CurProxI,NiouRand)
matrix(NiouDesPtI,Nprox+Nrand,1) # return only the indices of the perturbed points
} # end ZoupProxPerturb4Speed

# Hybrid-ESE with block inversion (see GridEseOptim)
# Use a "sequential criterion" 
# see BONUS in Test4IWS.R" ->  use for instance 'Seq.IMSE' or 'Seq.trcIMSE35'
GridEseOptimXtr<-function(IniId,Krit,Seq.Krit,Nprox,Nrand,ProxyMatGr,innerIt,it,Tol){  	 

  NbPert<-Nprox+Nrand
  CurDes<-IniId                      # Curent design
  np<-length(IniId)                  # Size of the design 
  bestDes<-CurDes                    # Best design (initialization)
  CimseD<-Krit(CurDes)               # Score of the curent design  
  bestVal<-CimseD                    # Best (i.e. minimun) IMSE
  Temperature<-0.005*CimseD          # Initial temperature
  NbEval<-1                          # Number of IMSE evaluation
  WhenBest<-NbEval                   # Indicate when the best value has been found
  TempRec<-Temperature               # Temperature recording
  
  for (q in 1:it){
     oldDes<-bestDes  ; oldVal<-bestVal                           
     ni<-0 ; na<-0 ;         # ni = number of improvements ; na = number of acceptances 
     for (count in 1:innerIt){          # Begin of the inner loop
     OrderMouv<-sample(1:np,np,replace=FALSE)
     for (k in 1:np){  
     EvoKrit<-Seq.Krit(CurDes[-OrderMouv[k]])		
     NewPts<-ZoupPerturb4Speed(OrderMouv[k],CurDes,np,Nprox,Nrand,Nq,ProxyMatGr)    # Design perturbations
     TryValS<-apply(NewPts,1,EvoKrit)                      # New designs IMSE
     NbEval<-NbEval+NbPert
     
     kb<-which.min(TryValS)
     kbDes<-CurDes ; kbDes[OrderMouv[k]]<-NewPts[kb,]
     kbVal<-TryValS[kb]
     Delta<-kbVal-CimseD
                                                
     if(Delta<=0){ # Always go if improvement
     	   CurDes<-kbDes
           CimseD<-kbVal
           na<-na+1           
             if(CimseD<=bestVal){  
                  bestDes<-CurDes            
                  bestVal<-CimseD 
                  WhenBest<-NbEval 
                  ni<-ni+1 } }   # end (if Delta<=0)
     else{ # Stochastic acceptance rule applied only to the best random perturbations 
     kbRand<-which.min(TryValS[-(1:Nprox)]) # Best rand value
     kbRandDes<-CurDes ; kbRandDes[OrderMouv[k]]<-NewPts[kbRand,]
     kbRandVal<-TryValS[Nprox+kbRand]
     Delta4Rand<-kbRandVal-CimseD
      
      if(Delta4Rand<=(Temperature*runif(1))){
     	   CurDes<-kbRandDes
           CimseD<-kbRandVal
           na<-na+1 
                      }   
                  } # end else   
  } # end for one OrderMouv
  } # end inner loop
  va<-na/(innerIt*np)    # va<-acceptance ratio
  vi<-ni/(innerIt*np)    # vi<-improvement ratio               
# Temperature update
  if (abs(bestVal-oldVal)<Tol){
              if(va>=0.1 & vi<=va)
                  {Temperature<-0.8*Temperature}
                       else {if (va>=0.1 & vi==va){  } 
                                      else {Temperature<-Temperature/0.8} }  }
      else { 
      if (va<=0.1){Temperature<-Temperature/0.7}
              else {Temperature<-Temperature*0.9} }
      TempRec<-c(TempRec,Temperature)                                    
                                               } # end outter loop 
list(Design=bestDes,Eval=NbEval,When=WhenBest,Temp=TempRec,Value=bestVal)
} # end GridEseOptimXtr



####################################################################
#####################################################
# Perturbation with LOW DISCREPANCY random designs
# the random perturbation consist in Nrand consecutive indices of the low discrepancy sequence
# (space filling random perturbation) 
ZoupPerturb4LowDis<-function(Pt2Pert,CurDesI,DesTai,Nprox,Nrand,Nq,ProxyMat){	 
	Npert<-Nprox+Nrand
# Point perturbations from Proxy Matrix   
	CurProxI<-ProxyMat[1:(Nprox+DesTai-1),CurDesI[Pt2Pert]]
	CurProxI<-setdiff(CurProxI,CurDesI)[1:Nprox] # leave out the new points which are already in the design 
	OccuPtI<-c(CurDesI,CurProxI)
# Random perturbations from Cov Matrix and weights 
KiKi<-sum(OccuPtI>=(Nq-Nrand))
IniNewRand<-sample((1:(Nq-Nrand-KiKi))[-OccuPtI],1,replace=FALSE)
NiouRand<-IniNewRand:(min(IniNewRand+Npert+DesTai,Nq))
NiouRand<-setdiff(NiouRand,OccuPtI)[1:Nrand]
NiouDesPtI<-c(CurProxI,NiouRand)

MpDes<-matrix(rep(CurDesI,Nprox+Nrand),DesTai,Npert)
MpDes[Pt2Pert,]<-NiouDesPtI

t(MpDes)  # lines of the returned matrix are the new design
} # end ZoupProxPerturb4LowDis
##############################################
# Hybrid-ESE algorithm using the perturbation 'ZoupPerturb4LowDis'
GridEseOptim4LowDis<-function(IniId,Krit,Nprox,Nrand,ProxyMatGr,innerIt,it,Tol,Nq){  	 

  NbPert<-Nprox+Nrand
  CurDes<-IniId                      # Curent design
  np<-length(IniId)                  # Size of the design 
  bestDes<-CurDes                    # Best design (initialization)
  CurSqr<-Krit(CurDes)               # Score of the curent design 
  bestVal<-CurSqr                    # Best (i.e. minimun) IMSE
  Temperature<-0.005*CurSqr          # Initial Temperature
  NbEval<-1                          # Number of IMSE evaluation
  WhenBest<-NbEval                   # Indicate when the best value has been found
  TempRec<-Temperature               # Temperature recording
  
  for (q in 1:it){
     oldDes<-bestDes  ; oldVal<-bestVal                           
     ni<-0 ; na<-0 ; 			         # ni = number of improvements ; na = number of acceptances 
     for (count in 1:innerIt){           # begin the inner loop
     OrderMouv<-sample(1:np,np,replace=FALSE)
     for (k in 1:np){  	
     MpDes<-ZoupPerturb4LowDis(OrderMouv[k],CurDes,np,Nprox,Nrand,Nq,ProxyMatGr)    # Design perturbations
     TryValS<-apply(MpDes,1,Krit)                      # New designs IMSE
     NbEval<-NbEval+NbPert
     
     kb<-which.min(TryValS)
     kbDes<-MpDes[kb,]
     kbVal<-TryValS[kb]
     Delta<-kbVal-CurSqr
                                                
     if(Delta<=0){ # Always go if improvement
     	   CurDes<-kbDes
           CurSqr<-kbVal
           na<-na+1           
             if(CurSqr<=bestVal){  
                  bestDes<-CurDes            
                  bestVal<-CurSqr 
                  WhenBest<-NbEval 
                  ni<-ni+1 } }   # end (if Delta<0)
     else{ # Stochastic acceptance rule applied only to the best random perturbations 
     kbRand<-which.min(TryValS[-(1:Nprox)]) # Best rand val
     kbRandDes<-MpDes[Nprox+kbRand,]
     kbRandVal<-TryValS[Nprox+kbRand]
     Delta4Rand<-kbRandVal-CurSqr
      
      if(Delta4Rand<=(Temperature*runif(1))){
     	   CurDes<-kbRandDes
           CurSqr<-kbRandVal
           na<-na+1 
           }   
                  } # end else               
  } # end for OrderMouv
  } # end inner loop
  va<-na/(innerIt*np)    # va<-acceptance ratio
  vi<-ni/(innerIt*np)    # vi<-improvement ratio               
# Temperature update
  if (abs(bestVal-oldVal)<Tol){
              if(va>=0.1 & vi<=va)
                  {Temperature<-0.8*Temperature}
                       else {if (va>=0.1 & vi==va){  } 
                                      else {Temperature<-Temperature/0.8} }  }
      else {
      if (va<=0.1){Temperature<-Temperature/0.7}
              else {Temperature<-Temperature*0.9} }
      TempRec<-c(TempRec,Temperature)                                    
                                               }
# Local descent (to finish)                                               
EndLocDesc<-BibiteDescent(bestDes,Krit,Nprox,ProxyMatGr)              
                                               
list(Design=EndLocDesc$Des,ESE.Design=bestDes,ESE.Eval=NbEval,ESE.When=WhenBest,Temp=TempRec,LocD.Info=EndLocDesc$Info)
} # end GridEseOptim4LowDis



#######################
###########################
#### Old ESE
# the stochastic aceptance rule is applied to the best of the perturbed design
OldGridEseOptim4LowDis<-function(IniId,Krit,Nprox,Nrand,ProxyMatGr,innerIt,it,Tol,Nq){  	 

  NbPert<-Nprox+Nrand
  CurDes<-IniId                      # Curent design
  np<-length(IniId)                  # Size of the design 
  bestDes<-CurDes                    # Best design (initialization)
  CurSqr<-Krit(CurDes)               # Score for the curent design 
  bestVal<-CurSqr                    # Best (i.e. minimun) IMSE
  Temperature<-0.005*CurSqr          # Initial Temperature
  NbEval<-1                          # Number of IMSE evaluation
  WhenBest<-NbEval                   # Indicate when the best value has been found
  TempRec<-Temperature               # Temperature recording
  
  for (q in 1:it){
     oldDes<-bestDes  ; oldVal<-bestVal                           
     ni<-0 ; na<-0 ;                     # ni = number of improvements ; na = number of acceptances 
     for (count in 1:innerIt){           # Begin of the inner loop
     OrderMouv<-sample(1:np,np,replace=FALSE)
     for (k in 1:np){  	
     MpDes<-ZoupPerturb4LowDis(OrderMouv[k],CurDes,np,Nprox,Nrand,Nq,ProxyMatGr)   # Design perturbations
     TryValS<-apply(MpDes,1,Krit)                      # New designs IMSE
     NbEval<-NbEval+NbPert
     
     kb<-which.min(TryValS)
     kbDes<-MpDes[kb,]
     kbVal<-TryValS[kb]
     Delta<-kbVal-CurSqr
                                                
     if(Delta<=(Temperature*runif(1))){ 
     	   CurDes<-kbDes
           CurSqr<-kbVal
           na<-na+1           
             if(CurSqr<=bestVal){  
                  bestDes<-CurDes            
                  bestVal<-CurSqr 
                  WhenBest<-NbEval 
                  ni<-ni+1 } }
  } # end for OrderMouv
  } # end inner loop
  va<-na/(innerIt*np)    # va<-acceptance ratio
  vi<-ni/(innerIt*np)    # vi<-improvement ratio               
# Temperature update
  if (abs(bestVal-oldVal)<Tol){
              if(va>=0.1 & vi<=va)
                  {Temperature<-0.8*Temperature}
                       else {if (va>=0.1 & vi==va){  } 
                                      else {Temperature<-Temperature/0.8} }  }
      else {  
      if (va<=0.1){Temperature<-Temperature/0.7}
              else {Temperature<-Temperature*0.9} }
      TempRec<-c(TempRec,Temperature)                                    
                                               }
list(Design=bestDes,Eval=NbEval,When=WhenBest,Temp=TempRec,Value=bestVal)
} # end OldGridEseOptim















