#######################################################
# Codes for the numerical experiments presented in the article 
# "Approximation of IMSE optimal designs via quadratures rules and spectral decomposition", 
# by B. Gauthier and L. Pronzato. 
# Encoded by B. Gauthier.  
# This work was supported by the project ANR-2011-IS01-001-01 DESIRE (DESIgns for spatial Random fiElds).
#########################################################
#########################################################
# APPLY THIS CODE LINE BY LINE ##########################
#########################################################
library(graphics) 		# for contour plot
library(rgl)    			# for 3d plod
#library(tikzDevice)		# for figures in TeX
library(gplots)  		# for color levels
#library(fOptions)      # for low discrepancy grid
library(microbenchmark)	# for computational time evaluation 

##########################################################
## Loading tool-functions 
source('Func4IWS.R')

#######################################################
#######################################################
# Two dimensional example 

# Kernel (Matern 3/2)
Lcar<-c(0.12,0.12)
Csqr3<-sqrt(3)/Lcar
KKernel<-function(x,y){ 
	d1p<-abs(x-y)*Csqr3
	prod(1+d1p)*exp(-sum(d1p)) }

# Square grid 37x37
# Grid Construction (axes discretization)
ndysX<-37
xdys<-DisInterEquiMid(0,1,ndysX)

ndysY<-37
ydys<-DisInterEquiMid(0,1,ndysY)

GridRectL<-list(xdys,ydys)

# Computation of the Grid :
GridInfo<-GridRectCons(GridRectL)
Gr.Q<-GridInfo$Grid     	 # Matrix of the grid points (rows are the points coordinates)
Nq<-nrow(Gr.Q)       	 # Total number of points in the grid

# Representation of the grid
par(mfrow=c(1,1))
plot(Gr.Q,pch=3)

# Grid weight assignation 
FunKTour<-function(r){ (1-r)^(3/2) *(1+cos(4*pi*min(r/0.5,1)))+0.2 }
FunKTourV<-Vectorize(FunKTour)
#rr<-seq(0,1/sqrt(2),,50)
#plot(rr,FunKTourV(rr),type='l')
#FunKTour(0.2437)

WouayteFk<-function(x){
y<-x-c(0.5,0.5)
r<-sqrt(t(y)%*%y)
FunKTour(r)	
}

# Vector of the grid weights
GrW<-apply(Gr.Q,1,WouayteFk)/Nq

# Representation (+ point at the middle of the grid)
filled.contour(xdys,ydys, z=matrix(GrW,ndysX,ndysY),col=colorpanel(12,'white','grey7'),nlevels=8,
                 plot.axes={axis(1); axis(2); points(Gr.Q, pch=3); points(Gr.Q[685,1],Gr.Q[685,2], pch=19,col='red') }) 
# sum of the grid weights
sum(GrW)

# IMSE 
# Covariance matrix of the grid points
MatQ<-GridMatCov(Gr.Q,KKernel)
# Diagonalisation of W^{1/2} %*% Q %*% W^{1/2}
EigQGr<-eigen(t(L.Diago.Mat.Mult(sqrt(GrW),t(L.Diago.Mat.Mult(sqrt(GrW),MatQ)))),symmetric=TRUE)  

# Matrix Sigma
MatSig<-MatQ%*%L.Diago.Mat.Mult(GrW,MatQ)

plot(EigQGr$values,pch=19,col='red')
# Trace
TAU<-sum(EigQGr$values)
sum(EigQGr$values[1])/TAU
sum(EigQGr$values[1:257])/TAU   # with 257, 99% of the variance...
sum(EigQGr$values[1:35])/TAU

##### 3d plot of the second eigenfunction
open3d()
persp3d(xdys,ydys, as.matrix(EigQGr$vectors[,2]), color='lightblue', back='lines',lwd=1,xlab = 'x', ylab = 'y', zlab = 'z')

# Test IMSE
Id<-sample(1:Nq,12,replace=FALSE) # random quadrature-design
TAU-sum(solve(MatQ[Id,Id])*MatSig[Id,Id])
# Direct evaluation (integral form)
Direct.IMSE(Plan=Gr.Q[Id,],Q.Gr=Gr.Q,Q.Wg=GrW,KKernel,With.trend=FALSE)

# IMSE criterion for quadrature-designs
IMSE<-function(Id) TAU-sum(solve(MatQ[Id,Id])*MatSig[Id,Id])

# truncated-IMSE
TrueP<-L.Diago.Mat.Mult(1/sqrt(GrW),EigQGr$vectors) #diag(1/sqrt(GrW))%*%EigQGr$vectors  
XYmatXT<-L.Diago.Mat.Mult(EigQGr$values,t(TrueP)) #t(TrueP%*%diag(EigQGr$values))

# truncated IMSE criterion (all truncation sets)
# Input:
# DesGrI ..... Indices of the quadrature-design
# Itrc ....... Truncation set
TruncatIMSE<-function(DesGrI,Itrc){ 
    KC<-MatQ[DesGrI,DesGrI] #Covariance matrix of the design
	CC<-chol(KC)                #K=t(CC)%*%CC
	CCm1<-backsolve(CC,diag(rep(1,length(DesGrI))),upper.tri=TRUE)
	MmT<-XYmatXT[Itrc,DesGrI]%*%CCm1
    TAU-sum(MmT^2) 
} #end TruncatIMSEGr
# Test truncated-IMSE
Id<-sample(1:Nq,12,replace=FALSE) # random 12-point quadrature-design
TruncatIMSE(Id,1:Nq)
IMSE(Id)

# Truncated-IMSE with fixed truncation set
Fixtrc35<-1:35
MatMtrc35<-t(XYmatXT[Fixtrc35,])%*%XYmatXT[Fixtrc35,]

trcIMSE35<-function(Id) TAU-sum(solve(MatQ[Id,Id])*MatMtrc35[Id,Id])
# Test
Id<-sample(1:Nq,12,replace=FALSE) # random quadrature-design
trcIMSE35(Id) ; TruncatIMSE(Id,Fixtrc35)

IMSE(Id)
TruncatIMSE(Id,1:Nq)
Direct.IMSE(Plan=Gr.Q[Id,],Q.Gr=Gr.Q,Q.Wg=GrW,KKernel,With.trend=FALSE)
################################## OPTIMIZATION
######## Proximity matrix (kernel metric)
# for non stationary kernel
# VV4diag<-diag(MatQ)
# mm4x<-matrix(0,Nq,Nq)
# for (k in 1:Nq) mm4x[k,]<-VV4diag
# ProxyMat<-apply(mm4x+t(mm4x)-2*MatQ,2,order)
ProxyMat<-apply(2-2*MatQ,2,order)
ProxyMat<-ProxyMat[-1,]

# Hybrid-ESE optimization
np<-33    # Search of a 33-point quadrature-design
Id<-sample(1:Nq,33,replace=FALSE) # random initial design
system.time(RezEz<-GridEseOptim(IniId=Id,Krit=trcIMSE35,Nprox=8,Nrand=120,ProxyMatGr=ProxyMat,innerIt=30,it=4,Tol=1e-10,Nq=Nq) )

# Best design (for comparison)
Best33PtSq37<-c(52, 60, 77, 109, 120, 140, 300, 315, 330, 384, 394, 520, 529, 545,
      554, 675, 685, 695, 816, 825, 841, 850, 976, 986, 1040, 1055, 1070, 1230,
      1250, 1261, 1293, 1310, 1318)
# Representation
filled.contour(xdys,ydys, z=matrix(GrW,ndysX,ndysY),col=colorpanel(12,'white','grey7'),nlevels=8,
                 plot.axes={axis(1); axis(2); points(Gr.Q, pch=3); 
                 points(Gr.Q[Best33PtSq37,], pch=19,col='green3');	# Best design
                 points(Gr.Q[Id,], pch=19,col='blue');              # initial design
                 points(Gr.Q[RezEz$Design,], pch=19,col='red') })   # obtained design  

# Efficiency
IMSE(Best33PtSq37)/IMSE(RezEz$Design)
trcIMSE35(Best33PtSq37)/trcIMSE35(RezEz$Design)
# Temperature evolution 
plot(RezEz$Temp)


#####################################
######## BONUS 
# Sequential criterion (block inversion of the covariance matrix),
# can be used when a part of the design is fixed
# IMSE
Seq.IMSE<-function(FixId){
MatKFixm1<-solve(MatQ[FixId,FixId])	
Evo.Krit<-function(NewId){
GroMm1<-solve(MatQ[NewId,NewId]-MatQ[NewId,FixId]%*%MatKFixm1%*%MatQ[FixId,NewId])
GT1<-MatKFixm1+MatKFixm1%*%MatQ[FixId,NewId]%*%GroMm1%*%MatQ[NewId,FixId]%*%MatKFixm1
GT2h<- -MatKFixm1%*%MatQ[FixId,NewId]%*%GroMm1
MatKAllm1<-rbind(cbind(GT1,GT2h),cbind(t(GT2h),GroMm1))
TAU-sum(MatKAllm1*MatSig[c(FixId,NewId),c(FixId,NewId)])	
}
return(Evo.Krit) }

# trcIMSE35
Seq.trcIMSE35<-function(FixId){
MatKFixm1<-solve(MatQ[FixId,FixId])	
Evo.Krit<-function(NewId){
GroMm1<-solve(MatQ[NewId,NewId]-MatQ[NewId,FixId]%*%MatKFixm1%*%MatQ[FixId,NewId])
GT1<-MatKFixm1+MatKFixm1%*%MatQ[FixId,NewId]%*%GroMm1%*%MatQ[NewId,FixId]%*%MatKFixm1
GT2h<- -MatKFixm1%*%MatQ[FixId,NewId]%*%GroMm1
MatKAllm1<-rbind(cbind(GT1,GT2h),cbind(t(GT2h),GroMm1))
TAU-sum(MatKAllm1*MatMtrc35[c(FixId,NewId),c(FixId,NewId)])	
}
return(Evo.Krit) }

# Test
IdFix<-sample(1:Nq,10,replace=FALSE) # random design
ImseFixPart<-Seq.IMSE(IdFix)
IdNew<-sample((1:Nq)[-IdFix],4,replace=FALSE)
ImseFixPart(IdNew)
IMSE(c(IdFix,IdNew))
########################## end BONUS 
##############################################


# Remark 1-pt design
OuanaPtdes<-apply(matrix(1:Nq,Nq,1),1,IMSE)
open3d()
persp3d(xdys,ydys, as.matrix(OuanaPtdes), color='lightblue', back='lines',lwd=1,xlab = 'x', ylab = 'y', zlab = 'z')

OuanaPtdesTrc3<-apply(matrix(1:Nq,Nq,1),1,TruncatIMSE,Itrc=1:3)
open3d()
persp3d(xdys,ydys, as.matrix(OuanaPtdesTrc3), color='lightblue', back='lines',lwd=1,xlab = 'x', ylab = 'y', zlab = 'z')

#tikz('Fig4CIOuana2.tex', standAlone = TRUE, width=11, height=6)
par(mfrow=c(1,2)) 
persp(xdys,ydys, TAU-matrix(OuanaPtdes,ndysX,ndysY), 
theta = 35, phi = 45, expand = 0.9, #shade=0.01, #col = "lightblue",
ticktype = 'detailed', xlab = '$x_{1}$', ylab = '$x_{2}$', zlab = '',axes=FALSE)
title(main='\\huge{$t\\mapsto C_{I}(\\mathbf{H}_{t})$}')

persp(xdys,ydys, TAU-matrix(OuanaPtdesTrc3,ndysX,ndysY), 
theta = 35, phi = 45, expand = 0.9, #shade=0.01, #col = "lightblue",
ticktype = 'detailed', xlab = '$x_{1}$', ylab = '$x_{2}$', zlab = '',axes=FALSE)
title(main='\\huge{$t\\mapsto C_{I_{trc}}(\\mathbf{H}_{t})$ for $n_{trc}=3$}')
#dev.off()

################################################
######### Dim2 Low Discrepancy
Nt<-2500
GrLowDiscrep<-runif.halton(n=Nt,dimension=2)  #library(fOptions) 
#GrLowDiscrep<-as.matrix(read.table(file='Data/LowDicr2500Gr4Test'))
dim(GrLowDiscrep)

par(mfrow=c(1,1)) 
plot(GrLowDiscrep,pch=3,col='grey',xlim=c(0,1),ylim=c(0,1))

GrWd<-apply(GrLowDiscrep,1,WouayteFk)/Nt
sum(GrWd)

# IMSE 
MatQd<-GridMatCov(GrLowDiscrep,KKernel)
EigQGrd<-eigen(t(L.Diago.Mat.Mult(sqrt(GrWd),t(L.Diago.Mat.Mult(sqrt(GrWd),MatQd)))),symmetric=TRUE)
MatSigd<-MatQd%*%L.Diago.Mat.Mult(GrWd,MatQd)
TAUd<-sum(EigQGrd$values)

IMSEd<-function(Id) TAUd-sum(solve(MatQd[Id,Id])*MatSigd[Id,Id])
# Test
Id<-sample(1:Nt,12,replace=FALSE)
TAUd-sum(solve(MatQd[Id,Id])*MatSigd[Id,Id])
Direct.IMSE(Plan=GrLowDiscrep[Id,],Q.Gr=GrLowDiscrep,Q.Wg=GrWd,KKernel,With.trend=FALSE)
IMSEd(Id)

# truncated-IMSE
TruePd<-L.Diago.Mat.Mult(1/sqrt(GrWd),EigQGrd$vectors)  
XYmatXTd<-L.Diago.Mat.Mult(EigQGrd$values,t(TruePd)) 

# Truncated-IMSE for various truncation set
TruncatIMSEd<-function(DesGrI,Itrc){ 
    KC<-MatQd[DesGrI,DesGrI] #Covariance matrix of the design
	CC<-chol(KC)                #K=t(CC)%*%CC
	CCm1<-backsolve(CC,diag(rep(1,length(DesGrI))),upper.tri=TRUE)
	MmT<-XYmatXTd[Itrc,DesGrI]%*%CCm1
    TAUd-sum(MmT^2) 
} #end TruncatIMSEGr

# Test	
TruncatIMSEd(Id,1:Nt)
IMSEd(Id)

# Fixed Truncation level
Fixtrc35<-1:35
MatMdtrc35<-t(XYmatXTd[Fixtrc35,])%*%XYmatXTd[Fixtrc35,]

trcIMSE35d<-function(Id) TAUd-sum(solve(MatQd[Id,Id])*MatMdtrc35[Id,Id])
trcIMSE35d(Id)
TruncatIMSEd(Id,Fixtrc35)

###### OPTIMIZATION

# Best IMSE design
OptDesMXLNtP33Tr<-read.table(file='Data/LowDicrNtWin33')$x  

plot(GrLowDiscrep,pch=3,col='grey',xlim=c(0,1),ylim=c(0,1))
points(GrLowDiscrep[OptDesMXLNtP33Tr,],pch=19,col='red')

######## Proximity matrix (kernel metric)
PrMatd<-apply(2-2*MatQd,2,order)
ProxyMatd<-PrMatd[-1,]

Id<-sample(1:Nt,33,replace=FALSE) # Random initial quadrature-design
system.time(RezEz<-GridEseOptim(IniId=Id,Krit=IMSEd,Nprox=12,Nrand=120,ProxyMatGr=ProxyMatd,innerIt=40,it=4,Tol=1e-6,Nq=Nt) )

plot(GrLowDiscrep,pch=3,col='grey',xlim=c(0,1),ylim=c(0,1))
points(GrLowDiscrep[OptDesMXLNtP33Tr,], pch=19,col='green3')
points(GrLowDiscrep[Id,], pch=19,col='blue')
points(GrLowDiscrep[RezEz$Design,], pch=19,col='red') 
points(GrLowDiscrep[RezEz35$Design,], pch=19,col='purple') 

# Efficiency
IMSEd(OptDesMXLNtP33Tr)/IMSEd(RezEz$Design)

# Local Descent from the IMSE optimum with the truncated criterion
BibiteDescent(IniId=OptDesMXLNtP33Tr,Krit=trcIMSE35d,Nprox=12,ProxyMatGr=ProxyMatd)

# Optimization of the truncated criterion
system.time(RezEz35<-GridEseOptim(IniId=Id,Krit=trcIMSE35d,Nprox=12,Nrand=120,ProxyMatGr=ProxyMatd,innerIt=40,it=4,Tol=1e-6,Nq=Nt) )
# Efficiency
IMSEd(OptDesMXLNtP33Tr)/IMSEd(RezEz35$Design)

# Best design for the truncated criterion (ntrc=35)
OptDesMXLNtP33Trc35<-read.table(file='Data/LowDicrNtWin33Trc35')$x  


plot(GrLowDiscrep,pch=3,col='grey',xlim=c(0,1),ylim=c(0,1))
points(GrLowDiscrep[OptDesMXLNtP33Tr,], pch=19,col='green3')
points(GrLowDiscrep[OptDesMXLNtP33Trc35,], pch=19,col='blue')

points(GrLowDiscrep[XXXc$Des,], pch=19,col='blue')
points(GrLowDiscrep[XXXc2$Des,], pch=19,col='purple')

# Representation
KiKouX<-setdiff(OptDesMXLNtP33Trc35,OptDesMXLNtP33Tr)
#tikz('33PtDesLowD.tex', standAlone = TRUE, width=7, height=4.2)
par(mfrow=c(1,1),mar=c(2.1,14.6,1.1,1.1), xpd=TRUE)    # Default c(5.1,4.1,6.1,5.1)

plot(GrLowDiscrep,pch=3,col='grey',xlim=c(0,1),ylim=c(0,1),xlab='',ylab='')
points(GrLowDiscrep[KiKouX,], pch=17,cex=1.3,col='green3')
points(GrLowDiscrep[OptDesMXLNtP33Tr,], pch=19,col='red')
legend(-0.62,0.62,  c( '\\large{Grid points}','\\large{IMSE optimal design}','\\large{IMSE$_{trc}$ optimal design}'),
        pch=c(3,19,17),y.intersp = 1.1,pt.cex=c(1,1,1.3),
        col=c('grey','red','green3')      )
#dev.off()


# Comparaison of the IMSE for a 74x74 square grid
ndysXs<-74
xdyss<-DisInterEquiMid(0,1,ndysXs)

ndysYs<-74
ydyss<-DisInterEquiMid(0,1,ndysYs)

	
GridRectL<-list(xdyss,ydyss)

# Computation of the Grid :
GridInfos<-GridRectCons(GridRectL)
Gr.Qs<-GridInfos$Grid     # Matrix of the grid points (rows are points coordinates)
Nqs<-nrow(Gr.Qs)        # Total number of points in the grid
par(mfrow=c(1,1))
plot(Gr.Qs,pch=3)
GrWs<-apply(Gr.Qs,1,WouayteFk)/Nqs

# Optimun for the square grid 37x37
Direct.IMSE(Plan=Gr.Q[Best33PtSq37,],Q.Gr=Gr.Qs,Q.Wg=GrWs,KKernel,With.trend=FALSE)
# IMSE optimum on the Low discreoancy grid
Direct.IMSE(Plan=GrLowDiscrep[OptDesMXLNtP33Tr,],Q.Gr=Gr.Qs,Q.Wg=GrWs,KKernel,With.trend=FALSE)
# trc-IMSE optimum on the Low discreoancy grid
Direct.IMSE(Plan=GrLowDiscrep[OptDesMXLNtP33Trc35,],Q.Gr=Gr.Qs,Q.Wg=GrWs,KKernel,With.trend=FALSE)



#########################################################################################
# EXAMPLE IN DIM 5
#########################################################################################
rm(list=ls(all=TRUE))   # clear all variable
# clear consol:   Cmd + alt + L (for Mac)
library(graphics)
library(rgl)
#library(tikzDevice)
library(gplots)
library(fOptions)
library(microbenchmark)

source('Func4IWS.R')
# Low discrepancy (mu-distributed) grid
Gr.Q<-as.matrix(read.table('Data/Pts4Dim5OPt2Gauss1GrAll5000'))

## Construction of the grid (BONUS)
# library(truncnorm)    # for truncated normal distribution
# QuantTrcNormFix<-function(p){ qtruncnorm(p, a=0, b=1, mean = 0.5, sd = 0.15) }
# Gr.Q<-runif.halton(n=5000,dimension=5)
# Gr.Q[,1]<-QuantTrcNormFix(Gr.Q[,1])
# Gr.Q[,2]<-QuantTrcNormFix(Gr.Q[,2])
## end BONUS 

dim(Gr.Q)
Nq<-5000
pairs(Gr.Q,pch=3)

# Kernel
Lcar<-c(0.22,0.52,0.52,0.52,0.22)
Csqr3<-sqrt(3)/Lcar
KKernel<-function(x,y){ 
	d1p<-abs(x-y)*Csqr3
	prod(1+d1p)*exp(-sum(d1p)) }

# Matrices for the IMSE and eigendecomposition
MatQ<-GridMatCov(Gr.Q,KKernel)
GrW<-rep(1/Nq,Nq)
EigQGr<-eigen((1/Nq)*MatQ,symmetric=TRUE)
MatSig<-MatQ%*%MatQ/Nq

plot(EigQGr$values,pch=19,col='red')
TAU<-sum(EigQGr$values)
sum(EigQGr$values[1])/TAU
sum(EigQGr$values[1:2])/TAU   # with 257, 99% of the variance...
sum(EigQGr$values[1:35])/TAU

IMSE<-function(Id) TAU-sum(solve(MatQ[Id,Id])*MatSig[Id,Id])
# Test
Id<-sample(1:Nq,12,replace=FALSE)
TAU-sum(solve(MatQ[Id,Id])*MatSig[Id,Id])
Direct.IMSE(Plan=Gr.Q[Id,],Q.Gr=Gr.Q,Q.Wg=GrW,KKernel,With.trend=FALSE)


## trc-IMSE
TrueP<-sqrt(Nq)*EigQGr$vectors   # 
XYmatXT<-L.Diago.Mat.Mult(EigQGr$values,t(TrueP)) 

TruncatIMSE<-function(DesGrI,Itrc){ 
    KC<-MatQ[DesGrI,DesGrI] #Covariance matrix of the design
	CC<-chol(KC)                #K=t(CC)%*%CC
	CCm1<-backsolve(CC,diag(rep(1,length(DesGrI))),upper.tri=TRUE)
	MmT<-XYmatXT[Itrc,DesGrI]%*%CCm1
    TAU-sum(MmT^2) 
} #end TruncatIMSE
# Test
TruncatIMSE(Id,1:Nq)   # no truncation
IMSE(Id)

# Optimization (local descent)
######## Proximity matrix (kernel metric)
ProxyMat<-apply(2-2*MatQ,2,order)
ProxyMat<-ProxyMat[-1,]

#######################################

np<-50 # design size
ndes<-100 # number of different initial low discrepancy design
Inipoint<-sample(1:(Nq-np),ndes,replace=FALSE)

IniTank<-matrix(0,ndes,np)  # matrix of the initial design (rows are designs)
for (ii in 1:ndes) IniTank[ii,]<-(Inipoint[ii]:(Inipoint[ii]+np-1))

# IMSE for the initial designs
RezIni<-apply(IniTank,1,IMSE)
par(mar=c(5.1,4.1,6.1,5.1))
boxplot(RezIni,horizontal=TRUE)

# Descent for the IMSE
system.time(AllRezDesc<-BibiteDescent(IniId=IniTank[1,],IMSE,Nprox=42,ProxyMatGr=ProxyMat))
RezDesc<-AllRezDesc$Des
for (kk in 2:100){
AllRezDesc<-BibiteDescent(IniId=IniTank[kk,],IMSE,Nprox=42,ProxyMatGr=ProxyMat)
RezDesc<-rbind(RezDesc,AllRezDesc$Des)	
}
RezI<-apply(RezDesc,1,IMSE)

boxplot(RezI,RezIni,horizontal=TRUE,outline=FALSE,axes=FALSE)


# Fix Truncation level
Fixtrc54<-1:54
MatMtrc54<-t(XYmatXT[Fixtrc54,])%*%XYmatXT[Fixtrc54,]

trcIMSE54<-function(Id) TAU-sum(solve(MatQ[Id,Id])*MatMtrc54[Id,Id])
# Test
trcIMSE54(Id)
TruncatIMSE(Id,Fixtrc54)

# Descent for the truncated-IMSE
AllRezDesc54<-BibiteDescent(IniId=IniTank[1,],trcIMSE54,Nprox=42,ProxyMatGr=ProxyMat)
RezDesc54<-AllRezDesc54$Des
for (kk in 2:100){
AllRezDesc54<-BibiteDescent(IniId=IniTank[kk,],trcIMSE54,Nprox=42,ProxyMatGr=ProxyMat)
RezDesc54<-rbind(RezDesc54,AllRezDesc54$Des)	
}
Rez54<-apply(RezDesc54,1,IMSE)

boxplot(RezI,Rez54)
bbmin<-min(c(RezI,Rez54))
bbmax<-max(c(RezI,Rez54))

min(Rez54) ; which.min(Rez54)
min(RezI) ; which.min(RezI)

# Graph for the article
#tikz('33PtLibertyGrb.tex', standAlone = TRUE, width=6, height=1.8)
par(mfrow=c(1,1),mar=c(2.1,4.1,2.1,5.1), xpd=TRUE)    # Default c(5.1,4.1,6.1,5.1)  c(bottom, left, top, right)
plot(10,10,type='p',pch=19,axes=FALSE,xlab='',ylab='',xlim=c(bbmin,0.42),ylim=c(0,2))
pos<-0.032
boxplot(RezI,Rez54,RezIni-pos,horizontal=TRUE,outline=FALSE,at=c(0.5,1.7,1),add=TRUE,axes=FALSE,names=c('IMSE','IMSE$_{trc}$','initial'))
axis(1,c(0.433,0.440,0.447,0.454)-pos,c(0.433,0.440,0.447,0.454),pos=0.5)
axis(1,c(0.389,0.394),c(0.389,0.394),pos=0)
text(0.412,2,'Initial designs')
text(0.412,1.7,'($50$-pt random low-discrepancy designs)')

text(0.391,1.1,'descent using IMSE')
text(0.3915,2.3,'descent using IMSE$_{trc}$')
#dev.off()

#Best design obtained with truncation
DesIII<-RezDesc54[which.min(Rez54),]

# Result used for the article
# write.table(RezI,'RezI')
# write.table(Rez54,'Rez54')
# write.table(DesIII,'DesIII')

#############################
# Hybrid-ESE from random design
Id<-sample(1:Nq,np,replace=FALSE)
system.time(RezEzI<-OldGridEseOptim4LowDis(IniId=Id,Krit=trcIMSE53,Nprox=42,Nrand=200,ProxyMatGr=ProxyMat,innerIt=50,it=2,Tol=1e-6,Nq=Nq) )
valEzI<-IMSE(RezEzI$Design)



# Pairs plot
N4plo<-1000  # number of quadrature points to be plotted
All<-data.frame(rbind(Gr.Q[1:N4plo,],Gr.Q[DesIII,]))
Wat<-c(rep('grey',N4plo),rep('red',np))
All$Col<- character(N4plo+np)
All$Col<-Wat
Wat<-c(rep(3,N4plo),rep(19,np))
All$Pch<-Wat
Wat<-c(rep(0.5,N4plo),rep(0.5,np))
All$Cex<-Wat

pairs(All[,1:5], col = All[,"Col"],las = 1, gap = 0, pch=All[,"Pch"],cex=All[,"Cex"],xlim=c(0,1),ylim=c(0,1),
       labels=c(expression(x[1]),expression(x[2]),expression(x[3]),expression(x[4]),expression(x[5])) )

# Only the design
pairs(Gr.Q[DesIII,], col = 'red',las = 1, gap = 0, pch=19,cex=0.7,xlim=c(0,1),ylim=c(0,1),
       labels=c(expression(x[1]),expression(x[2]),expression(x[3]),expression(x[4]),expression(x[5])) )









